local DROP_DOWN_WIDTH = 170
local VERTICAL_SPACE_BETWEEN_ELEMENTS = 15

local selectedMinILVL = nil
local selectedQuality = nil
local selectedNotifyMode = nil

--[[ Main Panel ]]--
local plHelperConfigFrame = CreateFrame('Frame', 'plHelperConfigFrame', InterfaceOptionsFramePanelContainer)
plHelperConfigFrame:Hide()
plHelperConfigFrame.name = PLH_LONG_ADDON_NAME

--[[ Title ]]--
local title = plHelperConfigFrame:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
title:SetPoint('TOPLEFT', 16, -16)
title:SetText(PLH_LONG_ADDON_NAME)

local author = plHelperConfigFrame:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
author:SetPoint('TOPRIGHT', plHelperConfigFrame, 'TOPRIGHT', -16, -24)
author:SetText('Author: ' .. PLH_AUTHOR_NAME)
InterfaceOptions_AddCategory(plHelperConfigFrame)

--[[ PLH_INCLUDE_BOE ]]--
local IncludeBOECheckbox = CreateFrame('CheckButton', 'PLHelperIncludeBOE', plHelperConfigFrame, 'OptionsBaseCheckButtonTemplate')
IncludeBOECheckbox:SetPoint('TOPLEFT', title, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
IncludeBOECheckbox:SetScript('OnShow', function(frame)
	frame:SetChecked(PLH_INCLUDE_BOE)
end)

local IncludeBOELabel = IncludeBOECheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
IncludeBOELabel:SetPoint('LEFT', IncludeBOECheckbox, 'RIGHT', 0, 0)
IncludeBOELabel:SetText('Include Bind on Equip items')

--[[ PLH_CURRENT_SPEC_ONLY ]]--
local CurrentSpecOnlyCheckbox = CreateFrame('CheckButton', 'PLHCurrentSpecOnly', plHelperConfigFrame, 'OptionsBaseCheckButtonTemplate')
CurrentSpecOnlyCheckbox:SetPoint('TOPLEFT', IncludeBOECheckbox, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
CurrentSpecOnlyCheckbox:SetScript('OnShow', function(frame)
	frame:SetChecked(PLH_CURRENT_SPEC_ONLY)
end)

local CurrentSpecOnlyLabel = CurrentSpecOnlyCheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
CurrentSpecOnlyLabel:SetPoint('LEFT', CurrentSpecOnlyCheckbox, 'RIGHT', 0, 0)
CurrentSpecOnlyLabel:SetText('Evaluate based on current spec only')

--[[ PLH_MIN_QUALITY ]]--
--[[ hiding for now; works, but propogrates taint due to Blizz issues with UIDROPDOWNMENU_MENU_LEVEL.  we'll just keep everyone at default (Rare)
local function GetItemQualityColorHex(quality)
	return '|c' .. select(4, GetItemQualityColor(quality))
end

if not QualityMenu then
    CreateFrame('Button', 'QualityMenu', plHelperConfigFrame, 'UIDropDownMenuTemplate')
end

QualityMenu:SetPoint('TOPLEFT', IncludeBOECheckbox, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
QualityMenu:Show()

local function GetQualityMenuText(index)
	local qualityName = _G['ITEM_QUALITY' .. index .. '_DESC']
	return GetItemQualityColorHex(index) .. qualityName
end

local function QualityMenu_OnClick(self, arg1, arg2, checked)
    selectedQuality = arg1
    UIDropDownMenu_SetText(QualityMenu, GetQualityMenuText(selectedQuality))
end

local function QualityMenu_Initialize(self, level)
    local info = UIDropDownMenu_CreateInfo()
    info.func = QualityMenu_OnClick
	for index = 0, 5 do  -- we only care about the first 6 qualities (through legendary)
		info.arg1 = index
		info.text = GetQualityMenuText(index)
		UIDropDownMenu_AddButton(info)
	end
end

QualityMenu:SetScript('OnShow', function(frame)
	selectedQuality = PLH_MIN_QUALITY
	UIDropDownMenu_SetText(QualityMenu, GetQualityMenuText(selectedQuality))
end)

UIDropDownMenu_Initialize(QualityMenu, QualityMenu_Initialize)
UIDropDownMenu_SetWidth(QualityMenu, DROP_DOWN_WIDTH);
UIDropDownMenu_JustifyText(QualityMenu, 'LEFT')

local QualityLabel = QualityMenu:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
QualityLabel:SetPoint('LEFT', QualityMenu, 'RIGHT', 0, 0)
QualityLabel:SetText('Minimum item quality')
]]--

-- [[ PLH_MIN_ILVL ]]
--[[ hiding for now; works, but propogrates taint due to Blizz issues with UIDROPDOWNMENU_MENU_LEVEL.  we'll just keep everyone at default (528)

local min_ilvl = {
	0,
	528,
	805,
	835,
	850,
	865,
	880
}

local min_ilvl_description = {
	'Unlimited (for testing purposes)',
	'Siege of Orgrimmar (first use of Personal Loot)',
	'Legion normal dungeon',
	'Legion LFR',
	'Legion normal raid',
	'Legion heroic raid',
	'Legion mythic raid'
}

if not MinILVLMenu then
    CreateFrame('Button', 'MinILVLMenu', plHelperConfigFrame, 'UIDropDownMenuTemplate')
end

MinILVLMenu:SetPoint('TOPLEFT', QualityMenu, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
MinILVLMenu:Show()

local function GetMinILVLMenuText(index)
	return min_ilvl[index] .. ' - ' .. min_ilvl_description[index]
end

local function MinILVLMenu_OnClick(self, arg1, arg2, checked)
    selectedMinILVL = arg1
    UIDropDownMenu_SetText(MinILVLMenu, GetMinILVLMenuText(selectedMinILVL))
end

local function MinILVLMenu_Initialize(self, level)
    local info = UIDropDownMenu_CreateInfo()
    info.func = MinILVLMenu_OnClick
	for i = 1, #min_ilvl do
		info.arg1 = i
		info.text = GetMinILVLMenuText(i)
		UIDropDownMenu_AddButton(info)
	end
end

-- returns the index within the array of the specific ilvl; ex: ilvl 528 returns index 1
local function GetMinILVLIndex(minILVL)
	local index = 1
	while min_ilvl[index] do
		if min_ilvl[index] == minILVL then
			return index
		else
			index = index + 1
		end
	end
	return 1  -- we couldn't find a match, so choose the first element in the list
end

MinILVLMenu:SetScript('OnShow', function(frame)
	selectedMinILVL = GetMinILVLIndex(PLH_MIN_ILVL)
	UIDropDownMenu_SetText(MinILVLMenu, GetMinILVLMenuText(selectedMinILVL))
end)

UIDropDownMenu_Initialize(MinILVLMenu, MinILVLMenu_Initialize)
UIDropDownMenu_SetWidth(MinILVLMenu, DROP_DOWN_WIDTH);
UIDropDownMenu_JustifyText(MinILVLMenu, 'LEFT')

local MinILVLLabel = MinILVLMenu:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
MinILVLLabel:SetPoint('LEFT', MinILVLMenu, 'RIGHT', 0, 0)
MinILVLLabel:SetText('Minimum item level')
]]--

--[[ PLH_NOTIFY_MODE ]]--
--[[ hiding for now; works, but propogrates taint due to Blizz issues with UIDROPDOWNMENU_MENU_LEVEL.  we'll use checkboxes instead
if not NotifyModeMenu then
    CreateFrame('Button', 'NotifyModeMenu', plHelperConfigFrame, 'UIDropDownMenuTemplate')
end

NotifyModeMenu:SetPoint('TOPLEFT', MinILVLMenu, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
NotifyModeMenu:Show()

local function NotifyModeMenu_OnClick(self, arg1, arg2, checked)
    selectedNotifyMode = arg1
    UIDropDownMenu_SetText(NotifyModeMenu, PLH_GetNotifyModeName(selectedNotifyMode))
end

local function NotifyModeMenu_Initialize(self, level)
    local info = UIDropDownMenu_CreateInfo()
    info.func = NotifyModeMenu_OnClick
	for index = 1, 3 do
		info.arg1 = index
		info.text = PLH_GetNotifyModeName(index)
		UIDropDownMenu_AddButton(info)
	end
end

NotifyModeMenu:SetScript('OnShow', function(frame)
	selectedNotifyMode = PLH_NOTIFY_MODE
	UIDropDownMenu_SetText(NotifyModeMenu, PLH_GetNotifyModeName(selectedNotifyMode))
end)

-- Final setup for dropdownmenu
UIDropDownMenu_Initialize(NotifyModeMenu, NotifyModeMenu_Initialize)
UIDropDownMenu_SetWidth(NotifyModeMenu, DROP_DOWN_WIDTH);
UIDropDownMenu_JustifyText(NotifyModeMenu, 'LEFT')

local NotifyModeLabel1 = NotifyModeMenu:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
NotifyModeLabel1:SetPoint('TOPLEFT', NotifyModeMenu, 'TOPRIGHT', 0, 0)
NotifyModeLabel1:SetWordWrap(true)
NotifyModeLabel1:SetJustifyH('LEFT')
NotifyModeLabel1:SetWidth(375)
NotifyModeLabel1:SetText('When tradeable loot is an upgrade for others in the group, Personal Loot Helper will...')

local NotifyModeLabel2 = NotifyModeMenu:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
NotifyModeLabel2:SetPoint('TOPLEFT', NotifyModeLabel1, 'BOTTOMLEFT', 20, 0)
NotifyModeLabel2:SetWordWrap(true)
NotifyModeLabel2:SetJustifyH('LEFT')
NotifyModeLabel2:SetWidth(355)
NotifyModeLabel2:SetText(
	'* Notify Self - ...notify only you\n' ..
	'* Notify Group - ...notify the group\n' ..
	'* Coordinate Rolls (group leader only) - ...notify the group and coordinate rolls if loot is an upgrade for more than one person. If you are not the group leader, this will behave the same as Notify Group.')
]]--

local NotifyGroupCheckbox = CreateFrame('CheckButton', 'PLHelperIncludeBOE', plHelperConfigFrame, 'OptionsBaseCheckButtonTemplate')
NotifyGroupCheckbox:SetPoint('TOPLEFT', CurrentSpecOnlyCheckbox, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS)
NotifyGroupCheckbox:SetScript('OnShow', function(frame)
	frame:SetChecked(PLH_NOTIFY_MODE == 2 or PLH_NOTIFY_MODE == 3)
end)
NotifyGroupCheckbox:SetScript('OnClick', function(frame)
	if NotifyGroupCheckbox:GetChecked() then
		PLH_CoordinateRollsCheckbox:Show()
		PLH_CoordinateRollsLabel1:Show()
		PLH_CoordinateRollsLabel2:Show()
	else
		PLH_CoordinateRollsCheckbox:Hide()
		PLH_CoordinateRollsLabel1:Hide()
		PLH_CoordinateRollsLabel2:Hide()
	end
end)

local NotifyGroupLabel1 = NotifyGroupCheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
NotifyGroupLabel1:SetPoint('LEFT', NotifyGroupCheckbox, 'RIGHT', 0, 0)
NotifyGroupLabel1:SetText('Notify Group')
NotifyGroupLabel1:SetWordWrap(true)
NotifyGroupLabel1:SetJustifyH('LEFT')
NotifyGroupLabel1:SetWidth(400)

PLH_CoordinateRollsCheckbox = CreateFrame('CheckButton', 'PLHelperIncludeBOE', plHelperConfigFrame, 'OptionsBaseCheckButtonTemplate')
PLH_CoordinateRollsCheckbox:SetPoint('TOPLEFT', NotifyGroupCheckbox, 'BOTTOMLEFT', 25, -VERTICAL_SPACE_BETWEEN_ELEMENTS / 2)
PLH_CoordinateRollsCheckbox:SetScript('OnShow', function(frame)
	frame:SetChecked(PLH_NOTIFY_MODE == 3)
end)


PLH_CoordinateRollsLabel1 = NotifyGroupCheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
PLH_CoordinateRollsLabel1:SetPoint('LEFT', PLH_CoordinateRollsCheckbox, 'RIGHT', 0, 0)
PLH_CoordinateRollsLabel1:SetText('Coordinate Rolls (group leader only)')
PLH_CoordinateRollsLabel1:SetWordWrap(true)
PLH_CoordinateRollsLabel1:SetJustifyH('LEFT')
PLH_CoordinateRollsLabel1:SetWidth(375)

PLH_CoordinateRollsLabel2 = NotifyGroupCheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
PLH_CoordinateRollsLabel2:SetPoint('TOPLEFT', PLH_CoordinateRollsLabel1, 'BOTTOMLEFT', 20, -10)
PLH_CoordinateRollsLabel2:SetText('Prompt for and manage rolls if loot is an upgrade for more than one person')
PLH_CoordinateRollsLabel2:SetWordWrap(true)
PLH_CoordinateRollsLabel2:SetJustifyH('LEFT')
PLH_CoordinateRollsLabel2:SetWidth(450)

plHelperConfigFrame:SetScript('OnShow', function(frame)
	if PLH_NOTIFY_MODE == 2 or PLH_NOTIFY_MODE == 3 then
		PLH_CoordinateRollsCheckbox:Show()
		PLH_CoordinateRollsLabel1:Show()
		PLH_CoordinateRollsLabel2:Show()
	else
		PLH_CoordinateRollsCheckbox:Hide()
		PLH_CoordinateRollsLabel1:Hide()
		PLH_CoordinateRollsLabel2:Hide()
	end
end)


--[[ Thank you message ]] --
local thank_you = plHelperConfigFrame:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
thank_you:SetPoint('BOTTOM', plHelperConfigFrame, 'BOTTOM', 0, 48)
thank_you:SetText('Thank you to all of the players who trade loot to others who can use it! Your generosity is part of what makes the WoW community great.\nYou rock!!!')
thank_you:SetWidth(300)
thank_you:SetWordWrap(true)

--[[ DEBUG ]]--
--[[
local DebugCheckbox = CreateFrame('CheckButton', 'PLHelperDebug', plHelperConfigFrame, 'OptionsBaseCheckButtonTemplate')
DebugCheckbox:SetPoint('TOPLEFT', NotifyGroupCheckbox, 'BOTTOMLEFT', 0, -VERTICAL_SPACE_BETWEEN_ELEMENTS * 6)
DebugCheckbox:SetScript('OnShow', function(frame)
	frame:SetChecked(PLH_DEBUG)
end)

local DebugLabel = DebugCheckbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
DebugLabel:SetPoint('LEFT', DebugCheckbox, 'RIGHT', 0, 0)
DebugLabel:SetText('Print debug messages')
]]--

--[[ Okay action ]]--
function plHelperConfigFrame.okay()
	xpcall(function()   -- wrapping the call so we can receive error messages for debugging
		PLH_INCLUDE_BOE = IncludeBOECheckbox:GetChecked()
		PLH_CURRENT_SPEC_ONLY = CurrentSpecOnlyCheckbox:GetChecked()
		PLH_MIN_QUALITY = 3  -- rare;  hardcoding until drop down taint issues are resolved
		PLH_MIN_ILVL = 528			-- hardcoding until drop down taint issues are resolved
--		PLH_MIN_QUALITY = selectedQuality
--		PLH_MIN_ILVL = min_ilvl[selectedMinILVL]
--		PLH_MIN_ILVL = editMinILVL:GetText()
--		if PLH_NOTIFY_MODE ~= selectedNotifyMode then
--			PLH_NOTIFY_MODE = selectedNotifyMode
--			PLH_SendStatusMessage(true)
--		end
		local selectedNotifyMode
		if NotifyGroupCheckbox:GetChecked() then
			if PLH_CoordinateRollsCheckbox:GetChecked() then
				selectedNotifyMode = 3
			else
				selectedNotifyMode = 2
			end
		else
			selectedNotifyMode = 1
		end
		if PLH_NOTIFY_MODE ~= selectedNotifyMode then
			PLH_NOTIFY_MODE = selectedNotifyMode
			PLH_SendStatusMessage(true)
		end
		
--		PLH_DEBUG = DebugCheckbox:GetChecked()
	end, geterrorhandler())
end

--[[ SAMPLE edit box; not clean
-- Minimum iLVL Editbox
local editMinILVL = CreateFrame('EditBox', 'PLHelperMinILVL', plHelperConfigFrame, 'InputBoxTemplate')
editMinILVL:SetPoint('TOPLEFT', 16, -112)
editMinILVL:SetWidth(100)
editMinILVL:SetHeight(1)
editMinILVL:SetNumeric(true)
editMinILVL:SetMaxLetters(3)
editMinILVL:SetScript('OnShow', function(frame)
	frame:SetText(PLH_MIN_ILVL)
end)
local editMinILVLText = editMinILVL:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
editMinILVLText:SetPoint('LEFT', editMinILVL, 'RIGHT', 0, 1)
editMinILVLText:SetJustifyH('LEFT')
editMinILVLText:SetWordWrap(true)
editMinILVLText:SetWidth(560)
editMinILVLText:SetText('Min ILVL for rolls')
--]]


--labelIncludeBOECheckbox:SetJustifyH('LEFT')
--labelIncludeBOECheckbox:SetWordWrap(true)
--labelIncludeBOECheckbox:SetWidth(560)

